#include "CConfigJogDialog.h"
#include <QtWidgets>
#include "COptions.h"
#include "DobotClient.h"
#include"MainWindow.h"
CConfigJogDialog::CConfigJogDialog(QWidget *parent) : QDialog(parent)
{
    setWindowTitle("参数设置");
    setFixedSize(412, 342);
    initLayout();
}

CConfigJogDialog::~CConfigJogDialog()
{

}

void CConfigJogDialog::initLayout(void)
{
    static const char *strLut[] = {
        "线速度",
        "线加速度",
        "关节角速度",
        "关节角加速度",
        "调节速度",
        "调节加速度",
        "速度百分比"
    };
    static const char *unitLut[] = {
        "mm/s",
        "mm/(s·s)",
        "°/s",
        "°/(s·s)",
        "°/s",
        "°/(s·s)",
        "%"
    };
    static const char *toolTipLut[] = {
        "Velocity of single axis",
        "Acceleration of single axis",
        "Velocity of single joint",
        "Acceleration of single joint",
        "Velocity of the servo",
        "Acceleration of the servo",
        "Ratio of the velocity"
    };

    QFont font("微软雅黑", 8);

    COptions::JogParams jogParams;
    COptions::instance()->getJogParams(&jogParams);
    float *params = (float *)&jogParams;

    int i = 0;
    for (i = 0; i < 7; i++) {
        QLabel *label = new QLabel(this);
        label->move(10, 10 + i * 40);
        label->resize(150, 30);
        label->setFont(font);
        label->setText(strLut[i]);

        QLineEdit *lineEdit = new QLineEdit(this);
        lineEdit->move(180, 10 + i * 40);
        lineEdit->resize(100, 30);
        lineEdit->setFont(font);
        lineEdit->setToolTip(toolTipLut[i]);
        lineEdit->setText(QString("%1").arg(params[i]));
        paramWidgetList << lineEdit;

        label = new QLabel(this);
        label->move(300, 10 + i * 40);
        label->resize(50, 20);
        label->setFont(font);
        label->setText(unitLut[i]);
    }

    QPushButton *sendBtn = new QPushButton(this);
    sendBtn->move(280, 10 + i * 40);
    sendBtn->resize(120, 50);
    sendBtn->setFont(font);
    sendBtn->setText("确定");
    connect(sendBtn, SIGNAL(clicked()), this, SLOT(onSendBtnClicked()));





}

void CConfigJogDialog::onSendBtnClicked(void)
{
    COptions::JogParams jogParams;
    float *params = (float *)&jogParams;

    int i = 0;
    for (i = 0; i < 7; i++) {
        QLineEdit *lineEdit = (QLineEdit *)paramWidgetList.at(i);
        params[i] = lineEdit->text().toFloat();
    }

    COptions::instance()->setJogParams(&jogParams);
    DobotClient::instance()->configJogParams(
                jogParams.jointVelocity,
                jogParams.jointAcceleration,
                jogParams.servoVelocity,
                jogParams.servoAcceleration,
                jogParams.linearVelocity,
                jogParams.linearAcceleration
                );
   COptions::instance()->setJogVelocityRatio(params[6]);
   DobotClient::instance()->setVelocityRatio(params[6]);

    accept();
}
