#include "MainWindow.h"
#include <QtWidgets>
#include <QDebug>
#include "DobotClient.h"
#include "CConfigJogDialog.h"
#include "CConfigPlayBackDialog.h"
#include "CPlayBackManager.h"
#include "COptions.h"
#include <QProcess>
#include <QApplication>

MainWindow::MainWindow(QWidget *parent)
    : QDialog(parent)
{
#ifndef QT_DEBUG
    // Start the process
    QProcess *process = new QProcess(this);
    process->setProgram(QString("%1/DobotServer.exe").arg(qApp->applicationDirPath()));
    process->start();

    connect(qApp, SIGNAL(aboutToQuit()), process, SLOT(kill()));
#endif

    widget[0] = new QWidget();
    widget[1] = new QWidget();
    widget[0]->setFixedSize(800, 290);
    widget[1]->setFixedSize(800, 300);
    initJogUI(widget[0]);
    initPlayBackUI(widget[1]);

    QVBoxLayout *layout = new QVBoxLayout();
    layout->addWidget(widget[0]);
    layout->addWidget(widget[1]);
    layout->setSizeConstraint(QLayout::SetFixedSize);
    setLayout(layout);

    connect(DobotClient::instance(), SIGNAL(newFrame(QVector<unsigned char>)),
            this, SLOT(onNewFrame(QVector<unsigned char>)));

    // Init the parameters!
    CConfigJogDialog jogParams;
    jogParams.onSendBtnClicked();
    CConfigPlayBackDialog playBackParams;
    playBackParams.onSendBtnClicked();

}

MainWindow::~MainWindow()
{

}

void MainWindow::initJogUI(QWidget *widget)
{
    QGroupBox *group = new QGroupBox("操作", widget);
    group->move(10, 10);
    group->resize(780, 280);
    group->setStyleSheet("QGroupBox{border:1px solid #303030;border-radius:5px;}");

    int baseX = 10 + 10, baseY = 10 + 30;
    QFont font(QFont("微软雅黑", 8));

    QGroupBox *jogGroup = new QGroupBox("点动控制", widget);
    jogGroup->move(baseX + 10, baseY);
    jogGroup->resize(475+35, 230);
    jogGroup->setStyleSheet("QGroupBox{border:1px solid #303030;border-radius:5px;}");


    QComboBox *jogMode = new QComboBox(widget);
    jogMode->setObjectName("jogMode");
    jogMode->move(baseX + 50+25, baseY + 30);
    jogMode->resize(90, 50);
    jogMode->addItem("绝对坐标");
    jogMode->addItem("关节角度");

    // The buttons
    QPushButton *baseAngleAddBtn = new QPushButton(widget);
    baseAngleAddBtn->setObjectName("baseAngleAddBtn");
    baseAngleAddBtn->move(baseX + 300-90+25, baseY + 30);
    baseAngleAddBtn->resize(90, 50);

    QPushButton *baseAngleSubBtn = new QPushButton(widget);
    baseAngleSubBtn->setObjectName("baseAngleSubBtn");
    baseAngleSubBtn->move(baseX + 520-180+25, baseY + 30);
    baseAngleSubBtn->resize(90, 50);

    QPushButton *longArmAddBtn = new QPushButton(widget);
    longArmAddBtn->setObjectName("longArmAddBtn");
    longArmAddBtn->move(baseX + 300-90+25, baseY + 90);
    longArmAddBtn->resize(90, 50);

    QPushButton *longArmSubBtn = new QPushButton(widget);
    longArmSubBtn->setObjectName("longArmSubBtn");
    longArmSubBtn->move(baseX + 520-180+25, baseY + 90);
    longArmSubBtn->resize(90, 50);

    QPushButton *shortArmAddBtn = new QPushButton(widget);
    shortArmAddBtn->setObjectName("shortArmAddBtn");
    shortArmAddBtn->move(baseX + 300-90+25, baseY + 150);
    shortArmAddBtn->resize(90, 50);

    QPushButton *shortArmSubBtn = new QPushButton(widget);
    shortArmSubBtn->setObjectName("shortArmSubBtn");
    shortArmSubBtn->move(baseX + 520-180+25, baseY + 150);
    shortArmSubBtn->resize(90, 50);


    COptions::JogParams jogParams;
    COptions::instance()->getJogParams(&jogParams);

    //QLabel *label = new QLabel(widget);
   // label->move(baseX + 300-240, baseY + 220);
   // label->resize(120, 30);
   // label->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
   // label->setText("速度百分比");

    QSlider *slider = new QSlider(widget);
    slider->move(baseX + 855-230-70+30, baseY + 85);
    slider->resize(30, 100);
    slider->setOrientation(Qt::Vertical);
    slider->setRange(-200, 200);
    slider->setValue(0);

    // RHead
    QSpinBox *rHead = new QSpinBox(widget);
    rHead->setObjectName("rHead");
    rHead->move(620+35, baseY + 5+90+20);
    rHead->resize(90, 30);
    rHead->setFont(font);
    rHead->setRange(-200,200);
    rHead->setAlignment(Qt::AlignCenter);
    rHead->setReadOnly(true);
    rHead->setStyleSheet("background-color:#e0e0e0");
    //rHead->setText("0");

   // QSpinBox *spinBox = new QSpinBox(widget);
  //  spinBox->move(baseX + 620-240, baseY + 220);
  ////  spinBox->resize(50, 30);
  //  spinBox->setRange(1, 100);
  //  spinBox->setValue((int)jogParams.velocityRatio);

  //  connect(slider, SIGNAL(valueChanged(int)), spinBox, SLOT(setValue(int)));
      connect(rHead, SIGNAL(valueChanged(int)), slider, SLOT(setValue(int)));
  //  connect(slider, SIGNAL(valueChanged(int)), this, SLOT(onJogVelocityRatioChanged(int)));

    // End type
    QGroupBox *endTypeGroup = new QGroupBox("单独控制", widget);
    endTypeGroup->move(baseX + 775-200-70+35, baseY);
    endTypeGroup->resize(330-120, 230);
    endTypeGroup->setStyleSheet("QGroupBox{border:1px solid #303030;border-radius:5px;}");



    // Laser
    QCheckBox *laser = new QCheckBox(widget);
    laser->setObjectName("laser");
    laser->move(baseX + 855-230-70+35, baseY + 130-100);
    laser->resize(180, 20);
    laser->setText("喷头光源同步开关");
    laser->setFont(font);
    laser->setChecked(false);

    QPushButton *gripperAddBtn = new QPushButton(widget);
    gripperAddBtn->setObjectName("gripperAddBtn");
    gripperAddBtn->move(620+35, baseY + 60);
    gripperAddBtn->resize(90, 50);
    gripperAddBtn->setText("喷头开关");

    QPushButton *gripperSubBtn = new QPushButton(widget);
    gripperSubBtn->setObjectName("gripperSubBtn");
    gripperSubBtn->move(620+35, baseY + 150);
    gripperSubBtn->resize(90, 50);
    gripperSubBtn->setText("光源开关");

    // Config button
    QPushButton *configBtn = new QPushButton(widget);
    configBtn->setObjectName("configBtn");
    configBtn->move(baseX + 50+25, baseY + 150);
    configBtn->resize(90, 50);
   // configBtn->setFont(font);
    configBtn->setText("参数设置");


    // Init the connects
    connect(jogMode, SIGNAL(currentIndexChanged(int)), this, SLOT(onJogModeChanged(int)));
    onJogModeChanged(0);

    QSignalMapper *signalMapper = new QSignalMapper(this);
    connect(baseAngleAddBtn, SIGNAL(pressed()), signalMapper, SLOT(map()));
    connect(baseAngleSubBtn, SIGNAL(pressed()), signalMapper, SLOT(map()));
    connect(longArmAddBtn, SIGNAL(pressed()), signalMapper, SLOT(map()));
    connect(longArmSubBtn, SIGNAL(pressed()), signalMapper, SLOT(map()));
    connect(shortArmAddBtn, SIGNAL(pressed()), signalMapper, SLOT(map()));
    connect(shortArmSubBtn, SIGNAL(pressed()), signalMapper, SLOT(map()));


    connect(gripperAddBtn, SIGNAL(pressed()), signalMapper, SLOT(map()));
    connect(gripperSubBtn, SIGNAL(pressed()), signalMapper, SLOT(map()));

    signalMapper->setMapping(baseAngleAddBtn, 0);
    signalMapper->setMapping(baseAngleSubBtn, 1);
    signalMapper->setMapping(longArmAddBtn, 2);
    signalMapper->setMapping(longArmSubBtn, 3);
    signalMapper->setMapping(shortArmAddBtn, 4);
    signalMapper->setMapping(shortArmSubBtn, 5);
    signalMapper->setMapping(gripperAddBtn, 8);
    signalMapper->setMapping(gripperSubBtn, 9);

    connect(signalMapper, SIGNAL(mapped(int)), this, SLOT(onJogCtrlBtnPressed(int)));

    connect(baseAngleAddBtn, SIGNAL(released()), this, SLOT(onJogCtrlBtnReleased()));
    connect(baseAngleSubBtn, SIGNAL(released()), this, SLOT(onJogCtrlBtnReleased()));
    connect(longArmAddBtn, SIGNAL(released()), this, SLOT(onJogCtrlBtnReleased()));
    connect(longArmSubBtn, SIGNAL(released()), this, SLOT(onJogCtrlBtnReleased()));
    connect(shortArmAddBtn, SIGNAL(released()), this, SLOT(onJogCtrlBtnReleased()));
    connect(shortArmSubBtn, SIGNAL(released()), this, SLOT(onJogCtrlBtnReleased()));
    connect(gripperAddBtn, SIGNAL(released()), this, SLOT(onJogCtrlBtnReleased()));
    connect(gripperSubBtn, SIGNAL(released()), this, SLOT(onJogCtrlBtnReleased()));
    onJogCtrlBtnReleased();


    //connect(grab, SIGNAL(toggled(bool)), this, SLOT(onGrabToggled(bool)));
    connect(laser, SIGNAL(toggled(bool)), this, SLOT(onLaserToggled(bool)));

    connect(configBtn, SIGNAL(clicked()), this, SLOT(onJogConfigBtnClicked()));

    //connect(userDefBtn, SIGNAL(clicked()), this, SLOT(onUserDefBtnClicked()));

}

void MainWindow::initPlayBackUI(QWidget *widget)
{
    QGroupBox *group = new QGroupBox("示教再现", widget);
    group->move(10, 8);
    group->resize(780, 290);
    group->setStyleSheet("QGroupBox{border:1px solid #303030;border-radius:5px;}");

    int baseX = 10 + 10, baseY = 8 + 30;
    QFont font(QFont("微软雅黑", 8));

    QGroupBox *Lianxu = new QGroupBox("连续运动", widget);
    Lianxu->move(baseX + 10, baseY);
    Lianxu->resize(475+35, 250);
    Lianxu->setStyleSheet("QGroupBox{border:1px solid #303030;border-radius:5px;}");

    QGroupBox *feedback = new QGroupBox("反馈信息", widget);
    feedback->move(baseX + 775-200-70+35, baseY);
    feedback->resize(330-120, 250);
    feedback->setStyleSheet("QGroupBox{border:1px solid #303030;border-radius:5px;}");


    // X
    QLabel *xLabel = new QLabel(widget);
    xLabel->move(535+35, baseY + 5+15);
    xLabel->resize(65, 30);
    xLabel->setFont(font);
    xLabel->setAlignment(Qt::AlignRight);
    xLabel->setText("X:");

    QLineEdit *x = new QLineEdit(widget);
    x->setObjectName("x");
    x->move(620+35, baseY + 5+15);
    x->resize(90, 30);
    x->setFont(font);
    x->setAlignment(Qt::AlignCenter);
    x->setReadOnly(true);
    x->setStyleSheet("background-color:#e0e0e0");
    x->setText("0");

    // Y
    QLabel *yLabel = new QLabel(widget);
    yLabel->move(535+35, baseY + 5+30+15);
    yLabel->resize(65, 30);
    yLabel->setFont(font);
    yLabel->setAlignment(Qt::AlignRight);
    yLabel->setText("Y:");

    QLineEdit *y = new QLineEdit(widget);
    y->setObjectName("y");
    y->move(620+35, baseY + 5+30+15);
    y->resize(90, 30);
    y->setFont(font);
    y->setAlignment(Qt::AlignCenter);
    y->setReadOnly(true);
    y->setStyleSheet("background-color:#e0e0e0");
    y->setText("0");

    // Z
    QLabel *zLabel = new QLabel(widget);
    zLabel->move(535+35, baseY + 5+60+15);
    zLabel->resize(65, 30);
    zLabel->setFont(font);
    zLabel->setAlignment(Qt::AlignRight);
    zLabel->setText("Z:");

    QLineEdit *z = new QLineEdit(widget);
    z->setObjectName("z");
    z->move(620+35, baseY + 5+60+15);
    z->resize(90, 30);
    z->setFont(font);
    z->setAlignment(Qt::AlignCenter);
    z->setReadOnly(true);
    z->setStyleSheet("background-color:#e0e0e0");
    z->setText("0");




    // IsGrab
    QLabel *IsGrabLabel = new QLabel(widget);
    IsGrabLabel->move(525+35, baseY + 5+90+15);
    IsGrabLabel->resize(75, 30);
    IsGrabLabel->setFont(font);
    IsGrabLabel->setAlignment(Qt::AlignRight);
    IsGrabLabel->setText("喷头光源开关:");


    QLineEdit *isGrab = new QLineEdit(widget);
    isGrab->setObjectName("isGrab");
    isGrab->move(620+35, baseY + 5+90+15);
    isGrab->resize(130-40, 30);
    isGrab->setFont(font);
    isGrab->setAlignment(Qt::AlignCenter);
    isGrab->setReadOnly(true);
    isGrab->setStyleSheet("background-color:#e0e0e0");
    isGrab->setText("false");
    // BaseAngle
    QLabel *BaseAngleLabel = new QLabel(widget);
    BaseAngleLabel->move(535+35, baseY + 5+120+15);
    BaseAngleLabel->resize(65, 30);
    BaseAngleLabel->setFont(font);
    BaseAngleLabel->setAlignment(Qt::AlignRight);
    BaseAngleLabel->setText("底座角度:");

    QLineEdit *baseAngle = new QLineEdit(widget);
    baseAngle->setObjectName("baseAngle");
    baseAngle->move(620+35, baseY + 5+120+15);
    baseAngle->resize(130-40, 30);
    baseAngle->setFont(font);
    baseAngle->setAlignment(Qt::AlignCenter);
    baseAngle->setReadOnly(true);
    baseAngle->setStyleSheet("background-color:#e0e0e0");
    baseAngle->setText("0");
    // LongArmAngle
    QLabel *LongArmAngleleLabel = new QLabel(widget);
    LongArmAngleleLabel->move(535+35, baseY + 5+150+15);
    LongArmAngleleLabel->resize(65, 30);
    LongArmAngleleLabel->setFont(font);
    LongArmAngleleLabel->setAlignment(Qt::AlignRight);
    LongArmAngleleLabel->setText("大臂角度:");

    QLineEdit *longArmAngle = new QLineEdit(widget);
    longArmAngle->setObjectName("longArmAngle");
    longArmAngle->move(620+35, baseY + 5+150+15);
    longArmAngle->resize(130-40, 30);
    longArmAngle->setFont(font);
    longArmAngle->setAlignment(Qt::AlignCenter);
    longArmAngle->setReadOnly(true);
    longArmAngle->setStyleSheet("background-color:#e0e0e0");
    longArmAngle->setText("0");
    // ShortArmAngle
    QLabel *ShortArmAngleleLabel = new QLabel(widget);
    ShortArmAngleleLabel->move(535+35, baseY + 5+180+15);
    ShortArmAngleleLabel->resize(65, 30);
    ShortArmAngleleLabel->setFont(font);
    ShortArmAngleleLabel->setAlignment(Qt::AlignRight);
    ShortArmAngleleLabel->setText("小臂角度:");

    QLineEdit *shortArmAngle = new QLineEdit(widget);
    shortArmAngle->setObjectName("shortArmAngle");
    shortArmAngle->move(620+35, baseY + 5+180+15);
    shortArmAngle->resize(130-40, 30);
    shortArmAngle->setFont(font);
    shortArmAngle->setAlignment(Qt::AlignCenter);
    shortArmAngle->setReadOnly(true);
    shortArmAngle->setStyleSheet("background-color:#e0e0e0");
    shortArmAngle->setText("0");



  // QLineEdit *pawAngle = new QLineEdit(widget);
  // pawAngle->setObjectName("pawAngle");
 // pawAngle->move(200, baseY -10);
 // pawAngle->resize(200, 30);
 //  pawAngle->setFont(font);
 // pawAngle->setAlignment(Qt::AlignCenter);
  //pawAngle->setReadOnly(true);
 // pawAngle->setStyleSheet("background-color:#e0e0e0");
 // pawAngle->setText("0");


  //Gripper
 //   QLineEdit *gripper = new QLineEdit(widget);
//   gripper->setObjectName("gripper");
 //  gripper->move(baseX + 35 + 6 * 90-90, baseY + 40);
 //   gripper->resize(130-40, 30);
//   gripper->setFont(font);
 ///  gripper->setAlignment(Qt::AlignCenter);
//   gripper->setReadOnly(true);
  // gripper->setStyleSheet("background-color:#e0e0e0");
 //  gripper->setText("0");
    // PauseTime
 //   QLineEdit *pauseTime = new QLineEdit(widget);
 //   pauseTime->setObjectName("pauseTime");
 //   pauseTime->move(baseX + 35 + 7 * 90-90, baseY + 40);
 //  pauseTime->resize(130-40, 30);
//   pauseTime->setFont(font);
//   pauseTime->setAlignment(Qt::AlignCenter);
//   pauseTime->setText("0");

    // Save
    QPushButton *saveBtn = new QPushButton(widget);
    saveBtn->setObjectName("saveBtn");
    saveBtn->move(baseX+45, baseY + 200);
    saveBtn->resize(70, 35);
    saveBtn->setFont(font);
    saveBtn->setText("存点");

    // SendAll
    QPushButton *sendAllBtn = new QPushButton(widget);
    sendAllBtn->setObjectName("sendAllBtn");
    sendAllBtn->move(baseX + 132.5, baseY + 200);
    sendAllBtn->resize(70, 35);
    sendAllBtn->setFont(font);
    sendAllBtn->setText("开始");

    // Stop
    QPushButton *stopBtn = new QPushButton(widget);
    stopBtn->setObjectName("stopBtn");
    stopBtn->move(baseX + 220, baseY + 200);
    stopBtn->resize(70, 35);
    stopBtn->setFont(font);
    stopBtn->setText("停止");



    QTableWidget *playBackTable = new QTableWidget(widget);
    playBackTable->setObjectName("playBackTable");
    playBackTable->move(baseX + 15, baseY +20);
    playBackTable->resize(1100-330-310+35, 220-50+5);
    playBackTable->setColumnCount(7);
    QStringList header;
    header << "喷头光源开关"<< "X" << "Y" << "Z" << "起始速度"<< "终止速度" << "最大速度";
    playBackTable->setHorizontalHeaderLabels(header);
    playBackTable->setSelectionBehavior(QAbstractItemView::SelectRows);
    playBackTable->setSelectionMode(QAbstractItemView::SingleSelection);
    playBackTable->setFocusPolicy(Qt::StrongFocus);

    for (int i = 0; i < 7; i++) {
        playBackTable->setColumnWidth(i, 63.5);
    }
    playBackTable->horizontalHeader()->setFont(QFont("微软雅黑", 8));
    playBackTable->horizontalHeader()->setStyleSheet("QHeaderView::section{border:1px solid #808080;color:blue;}");

    playBackTable->verticalHeader()->setFont(QFont("微软雅黑", 8));
    playBackTable->verticalHeader()->setStyleSheet("QHeaderView::section{border:1px solid #808080;color:blue;}");
    playBackTable->verticalHeader()->setFixedWidth(30);
    playBackTable->setGridStyle(Qt::SolidLine);
    playBackTable->setFont(QFont("微软雅黑", 8));

    COptions::PlayBackParams playBackParams;
    COptions::instance()->getPlayBackParams(&playBackParams);



    // Restore the playback points!
    QVector<DobotClient::PlayBackPoint> playBackPoints;
    CPlayBackManager::instance()->restore(playBackPoints);
    foreach(DobotClient::PlayBackPoint playBackPoint, playBackPoints) {
        insertPlayBackRow(playBackPoint);
    }

    // Init the connects
    connect(sendAllBtn, SIGNAL(clicked()), this, SLOT(onSendAllBtnClicked()));
    connect(stopBtn, SIGNAL(clicked()), this, SLOT(onStopBtnClicked()));

    connect(saveBtn, SIGNAL(clicked()), this, SLOT(insertPlayBackRow()));
   // connect(configBtn, SIGNAL(clicked()), this, SLOT(onPlayBackConfigBtnClicked()));
}

void MainWindow::insertPlayBackRow(const DobotClient::PlayBackPoint &playBackPoint)
{
    QTableWidget *playBackTable = widget[1]->findChild<QTableWidget *>("playBackTable");

    int rowCount = playBackTable->rowCount();
    playBackTable->insertRow(rowCount);

    // IsGrap
    QComboBox *isGrab = new QComboBox(this);
    isGrab->addItem("关");
    isGrab->addItem("开");
    isGrab->setCurrentIndex(playBackPoint.isGrab);
    playBackTable->setCellWidget(rowCount, 0, isGrab);
    // BaseAngle
    QLineEdit *baseAngle = new QLineEdit(this);
    baseAngle->setAlignment(Qt::AlignCenter);
    baseAngle->setText(QString("%1").arg(playBackPoint.baseAngle));
    playBackTable->setCellWidget(rowCount, 1, baseAngle);
    // LongArmAngle
    QLineEdit *longArmAngle = new QLineEdit(this);
    longArmAngle->setAlignment(Qt::AlignCenter);
    longArmAngle->setText(QString("%1").arg(playBackPoint.longArmAngle));
    playBackTable->setCellWidget(rowCount, 2, longArmAngle);
    // ShortArmAngle
    QLineEdit *shortArmAngle = new QLineEdit(this);
    shortArmAngle->setAlignment(Qt::AlignCenter);
    shortArmAngle->setText(QString("%1").arg(playBackPoint.shortArmAngle));
    playBackTable->setCellWidget(rowCount, 3, shortArmAngle);
    // PawAngle
    QLineEdit *pawAngle = new QLineEdit(this);
    pawAngle->setAlignment(Qt::AlignCenter);
    pawAngle->setText(QString("%1").arg(playBackPoint.pawAngle));
     playBackTable->setCellWidget(rowCount, 4, pawAngle);
    // Gripper
    QLineEdit *gripper = new QLineEdit(this);
    gripper->setAlignment(Qt::AlignCenter);
    gripper->setText(QString("%1").arg(playBackPoint.gripper));
    playBackTable->setCellWidget(rowCount, 5, gripper);
    // PauseTime
    QLineEdit *pauseTime = new QLineEdit(this);
    pauseTime->setAlignment(Qt::AlignCenter);
    pauseTime->setText(QString("%1").arg(playBackPoint.pauseTime));
    playBackTable->setCellWidget(rowCount, 6, pauseTime);

    playBackTable->scrollToBottom();
}

void MainWindow::getPlayBackPoints(QVector<DobotClient::PlayBackPoint> &playBackPoints)
{
    playBackPoints.clear();
    QTableWidget *playBackTable = widget[1]->findChild<QTableWidget *>("playBackTable");
    int rowCount = playBackTable->rowCount();

    for (int j = 0; j < rowCount; j++) {
        //QComboBox *motionStyle = (QComboBox *)playBackTable->cellWidget(j, 0);
        QComboBox *isGrab = (QComboBox *)playBackTable->cellWidget(j, 0);
        QLineEdit *baseAngle = (QLineEdit *)playBackTable->cellWidget(j, 1);
        QLineEdit *longArmAngle = (QLineEdit *)playBackTable->cellWidget(j, 2);
        QLineEdit *shortArmAngle = (QLineEdit *)playBackTable->cellWidget(j, 3);
        QLineEdit *pawAngle = (QLineEdit *)playBackTable->cellWidget(j, 4);
        QLineEdit *gripper = (QLineEdit *)playBackTable->cellWidget(j, 5);
        QLineEdit *pauseTime = (QLineEdit *)playBackTable->cellWidget(j, 6);

        DobotClient::PlayBackPoint playBackPoint;
        memset(&playBackPoint, 0, sizeof(playBackPoint));
        playBackPoint.isGrab = isGrab->currentIndex() == 1;
        playBackPoint.baseAngle = baseAngle->text().toFloat();
        playBackPoint.longArmAngle = longArmAngle->text().toFloat();
        playBackPoint.shortArmAngle = shortArmAngle->text().toFloat();
        playBackPoint.pawAngle = pawAngle->text().toFloat();
        playBackPoint.gripper = gripper->text().toFloat();
        playBackPoint.pauseTime = pauseTime->text().toFloat();

        playBackPoints.push_back(playBackPoint);
    }
}

void MainWindow::onJogModeChanged(int index)
{
    QPushButton *baseAngleAddBtn = widget[0]->findChild<QPushButton *>("baseAngleAddBtn");
    QPushButton *baseAngleSubBtn = widget[0]->findChild<QPushButton *>("baseAngleSubBtn");
    QPushButton *longArmAddBtn = widget[0]->findChild<QPushButton *>("longArmAddBtn");
    QPushButton *longArmSubBtn = widget[0]->findChild<QPushButton *>("longArmSubBtn");
    QPushButton *shortArmAddBtn = widget[0]->findChild<QPushButton *>("shortArmAddBtn");
    QPushButton *shortArmSubBtn = widget[0]->findChild<QPushButton *>("shortArmSubBtn");
  //  QPushButton *rHeadAddBtn = widget[0]->findChild<QPushButton *>("rHeadAddBtn");
  //  QPushButton *rHeadSubBtn = widget[0]->findChild<QPushButton *>("rHeadSubBtn");

    if (index == 1) {
        baseAngleAddBtn->setText("Joint1+");
        baseAngleSubBtn->setText("Joint1-");
        longArmAddBtn->setText("Joint2+");
        longArmSubBtn->setText("Joint2-");
        shortArmAddBtn->setText("Joint3+");
        shortArmSubBtn->setText("Joint3-");
    } else {
        baseAngleAddBtn->setText("X+");
        baseAngleSubBtn->setText("X-");
        longArmAddBtn->setText("Y+");
        longArmSubBtn->setText("Y-");
        shortArmAddBtn->setText("Z+");
        shortArmSubBtn->setText("Z-");
    }
    DobotClient::instance()->setJogMode(index);
}

void MainWindow::onJogCtrlBtnPressed(int index)
{
    switch (index) {
        case 0:
            DobotClient::instance()->addAxis1();
        break;
        case 1:
            DobotClient::instance()->subAxis1();
        break;
        case 2:
            DobotClient::instance()->addAxis2();
        break;
        case 3:
            DobotClient::instance()->subAxis2();
        break;
        case 4:
            DobotClient::instance()->addAxis3();
        break;
        case 5:
            DobotClient::instance()->subAxis3();
        break;
        case 6:
            DobotClient::instance()->addAxis4();
        break;
        case 7:
            DobotClient::instance()->subAxis4();
        break;
        case 8:
            DobotClient::instance()->addAxis5();
        break;
        case 9:
            DobotClient::instance()->subAxis5();
        break;
    }
}

void MainWindow::onJogCtrlBtnReleased(void)
{
    DobotClient::instance()->stop();
}

void MainWindow::onJogVelocityRatioChanged(int value)
{
    COptions::instance()->setJogVelocityRatio(value);
    DobotClient::instance()->setVelocityRatio(value);
}



void MainWindow::onLaserToggled(bool checked)
{
    DobotClient::instance()->enableLaser(checked);
}

void MainWindow::onJogConfigBtnClicked(void)
{
    CConfigJogDialog dlg;
    dlg.exec();
}


void MainWindow::insertPlayBackRow(void)
{
    DobotClient::PlayBackPoint playBackPoint;

    memset(&playBackPoint, 0, sizeof(playBackPoint));

    QLineEdit *isGrab = widget[1]->findChild<QLineEdit *>("isGrab");
    playBackPoint.isGrab = isGrab->text() == QString("Yes");

    QLineEdit *baseAngle = widget[1]->findChild<QLineEdit *>("x");
    playBackPoint.baseAngle = baseAngle->text().toFloat();

    QLineEdit *longArmAngle = widget[1]->findChild<QLineEdit *>("y");
    playBackPoint.longArmAngle = longArmAngle->text().toFloat();

    QLineEdit *shortArmAngle = widget[1]->findChild<QLineEdit *>("z");
    playBackPoint.shortArmAngle = shortArmAngle->text().toFloat();

    //QLineEdit *pawAngle = widget[1]->findChild<QLineEdit *>("rHead");
   // playBackPoint.pawAngle = pawAngle->text().toFloat();

   // QLineEdit *gripper = widget[1]->findChild<QLineEdit *>("gripper");
   // playBackPoint.gripper = gripper->text().toFloat();

   // QLineEdit *pauseTime = widget[1]->findChild<QLineEdit *>("pauseTime");
   // playBackPoint.pauseTime = pauseTime->text().toFloat();

    insertPlayBackRow(playBackPoint);

    QVector<DobotClient::PlayBackPoint> playBackPoints;
    getPlayBackPoints(playBackPoints);
    CPlayBackManager::instance()->store(playBackPoints);
}

void MainWindow::removePlayBackRow(void)
{
    QTableWidget *playBackTable = widget[1]->findChild<QTableWidget *>("playBackTable");
    if (not playBackTable) {
        return;
    }
    playBackTable->removeRow(playBackTable->currentRow());

    QVector<DobotClient::PlayBackPoint> playBackPoints;
    getPlayBackPoints(playBackPoints);
    CPlayBackManager::instance()->store(playBackPoints);
}

void MainWindow::onPlayBackRatioChanged(void)
{
    QSlider *slider1 = widget[1]->findChild<QSlider *>("VelocityRatio");
    QSlider *slider2 = widget[1]->findChild<QSlider *>("AccelerationRatio");
    float params1 = slider1->value();
    float params2 = slider2->value();

    COptions::instance()->setPlayBackVelocityParams(params1);
    COptions::instance()->setPlayBackAccelerationRatio(params2);
    DobotClient::instance()->configPlayBackDynamicParams(params1, params2);
}

void MainWindow::onSendAllBtnClicked(bool force)
{
    QTableWidget *playBackTable = widget[1]->findChild<QTableWidget *>("playBackTable");
    int rowCount = playBackTable->rowCount();

    if (rowCount < 1) {
        QMessageBox::warning(this, "Warning", "No enough points!!!");
        return;
    }
    bool ok;
    int loopTimes;

    if (force) {
        ok = true;
        loopTimes = 1;
    } else {
        loopTimes = QInputDialog::getInt(this, "Set loop times", "Loop times", 1, 1, 100000, 1, &ok, Qt::Dialog);
    }
    if (ok) {
        for (int i = 0; i < loopTimes; i++) {
            QVector<DobotClient::PlayBackPoint> playBackPoints;
            getPlayBackPoints(playBackPoints);
            foreach(DobotClient::PlayBackPoint playBackPoint, playBackPoints) {

                DobotClient::instance()->gotoXYZ(
                            playBackPoint.isGrab,
                            playBackPoint.baseAngle,
                            playBackPoint.longArmAngle,
                            playBackPoint.shortArmAngle,
                            playBackPoint.pawAngle,
                            playBackPoint.gripper,
                            playBackPoint.pauseTime);
            }
        }
    }
}

void MainWindow::onStopBtnClicked(void)
{
    DobotClient::instance()->stop();
}

void MainWindow::onPlayBackConfigBtnClicked(void)
{
    CConfigPlayBackDialog dlg;
    dlg.exec();
}

void MainWindow::keyPressEvent(QKeyEvent *keyEvent)
{
    if (keyEvent->key() != Qt::Key_Delete) {
        return;
    }
    removePlayBackRow();
}

void MainWindow::onNewFrame(const QVector<unsigned char> &frame)
{
    float *params = (float *)frame.data();

    QLineEdit *x = widget[1]->findChild<QLineEdit *>("x");
    x->setText(QString("%1").arg(params[0]));
    QLineEdit *y = widget[1]->findChild<QLineEdit *>("y");
    y->setText(QString("%1").arg(params[1]));
    QLineEdit *z = widget[1]->findChild<QLineEdit *>("z");
    z->setText(QString("%1").arg(params[2]));
   // QLineEdit *rHead = widget[0]->findChild<QLineEdit *>("rHead");
    //rHead->setText(QString("%1").arg((int)params[9]));
    QSpinBox *rHead = widget[0]->findChild<QSpinBox *>("rHead");
    rHead->setValue((int)params[9]);
   // QSlider *slider =widget[0]->findChild<QSlider *>("slider");
    //slider->setValue(0);
    //slider->setValue(0);

    QLineEdit *baseAngle = widget[1]->findChild<QLineEdit *>("baseAngle");
    baseAngle->setText(QString("%1").arg(params[4]));
    QLineEdit *longArmAngle = widget[1]->findChild<QLineEdit *>("longArmAngle");
    longArmAngle->setText(QString("%1").arg(params[5]));
    QLineEdit *shortArmAngle = widget[1]->findChild<QLineEdit *>("shortArmAngle");
    shortArmAngle->setText(QString("%1").arg(params[6]));
    QLineEdit *isGrab = widget[1]->findChild<QLineEdit *>("isGrab");
    if (params[8] != 0) {
        isGrab->setText("Yes");
    } else {
        isGrab->setText("No");
    }

}
